/* Virginia Tech Cognitive Radio Open Source Systems
 * Virginia Tech, 2009
 *
 * LICENSE INFORMATION GOES HERE
 */

/* DESCRIPTION OF FILE.
 */


#include <cstdlib>
#include <cstring>
#include <stdint.h>

#include "vtcross/common.h"
#include "vtcross/components.h"
#include "vtcross/containers.h"
#include "vtcross/debug.h"
#include "vtcross/error.h"
#include "vtcross/socketcomm.h"


PolicyEngine::PolicyEngine()
{
    LOG("Creating Policy Engine.\n");
    SML_present = false;
    control = new RemoteComponent;
    LoadPolicies();
}


PolicyEngine::~PolicyEngine()
{
    delete control;
}


PolicyEngine::PolicyEngine(const char* serverName, const char* portNumber, \
        const bool SML)
{
    LOG("Creating Policy Engine.\n");

    SML_present = SML;
    control = new RemoteComponent;
    control->serverName = std::string(serverName);
    control->serverPort = atoi(portNumber);

    control->socketFD = ClientSocket(serverName, portNumber);

    if(SML_present) {
        RegisterServices(control->socketFD);
        LOG("Policy Engine connected to SML at %s.\n", serverName);
    }
    else {
        RegisterComponent(control->socketFD);
        LOG("Policy Engine connected to shell at %s.\n", serverName);
    }

    LoadPolicies();
}


void 
PolicyEngine::GetRemoteComponentType(int32_t socketFD)
{
}


void 
PolicyEngine::WaitForSignal(int32_t socketFD)
{
    LOG("Policy Engine:: Waiting for Policy Check Request.\n");

    int32_t decision_array[10];
    struct Parameter pList[10];
    struct CE_Info ce_info;

    // TODO should this be a GetRequest call or something else?
    if(GetRequest(socketFD, pList, &ce_info)) {
        LOG("Policy Engine:: Validating Transmission Parameters.\n");
        ValidateParameters(pList, &ce_info, decision_array);    

        LOG("Policy Engine:: Sending Policy decision to Server.\n");
        SendPEDecision(socketFD, pList, &ce_info, decision_array);
    }
}


void 
PolicyEngine::Shutdown()
{
    if(SML_present)
        DeregisterServices(control->socketFD);
    else
        DeregisterComponent(control->socketFD);
}


void
PolicyEngine::Reset()
{
    LOG("Resetting Policy Engine.\n");
    SML_present = false;
    control = new RemoteComponent;
    LoadPolicies();
}


void
PolicyEngine::RegisterComponent(int32_t socketFD)
{
    SendMessage(socketFD, "register_engine_policy");
    LOG("Policy Engine:: Registration message sent.\n");

}


void 
PolicyEngine::DeregisterComponent(int32_t socketFD)
{
    SendMessage(socketFD, "deregister_engine_policy");
    LOG("Policy Engine:: Registration message sent.\n");
}


void 
PolicyEngine::RegisterServices(int32_t socketFD)
{
    LOG("Policy Engine:: Registering services.\n");
    SendMessage(socketFD, "register_service_pe_geo");
    SendMessage(socketFD, "register_service_pe_time");
    SendMessage(socketFD, "register_service_pe_spectrum");
    SendMessage(socketFD, "register_service_pe_spacial");
}


void 
PolicyEngine::DeregisterServices(int32_t socketFD)
{
    LOG("Policy Engine:: Deregistering services.\n");
    SendMessage(socketFD, "deregister_service_pe_geo");
    SendMessage(socketFD, "deregister_service_pe_time");
    SendMessage(socketFD, "deregister_service_pe_spectrum");
    SendMessage(socketFD, "deregister_service_pe_spacial");
}


void 
PolicyEngine::LoadPolicies()
{
    LOG("PolicyEngine:: Loading policies.\n");
}


void 
PolicyEngine::ReloadPolicies()
{
    LOG("PolicyEngine:: Reloading policies.\n");
}


void 
PolicyEngine::SendPEDecision(int32_t socketFD, struct Parameter pList[], \
        struct CE_Info *ce_info, int32_t decision_array[])
{
    char var[50];
 
    for (size_t i = 0; i < ce_info->numParameters; i++) {
        sprintf(var, "%i", decision_array[i]);
        SendMessage(socketFD, var);
    }
}


void 
PolicyEngine::ValidateParameters(struct Parameter pList[], \
        struct CE_Info *ce_info, int decision_array[])
{
    LOG("Policy Engine:: Policies Validated.\n");
    for (size_t i = 0; i < ce_info->numParameters; i++)
        decision_array[i] = 1;
}

